﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using PayPal.PayPalAPIInterfaceService;
using PayPal.PayPalAPIInterfaceService.Model;
using log4net;

// The SetExpressCheckout API operation initiates an Express Checkout transaction.
namespace PayPalCredit
{
    public partial class SetExpressCheckout : System.Web.UI.Page
    {
        public readonly static string ReturnUrl;
        public readonly static string CancelUrl;
        public readonly static string LogoUrl;
        public readonly static string SellerEmail;
        public readonly static string RedirectUrl;

        static SetExpressCheckout()
        {
            // Load the log4net configuration settings from Web.config or App.config
            log4net.Config.XmlConfigurator.Configure();

            string schemeHost = HttpContext.Current.Request.Url.GetLeftPart(UriPartial.Authority);

            //Load values from web.config (configuration file)
            var config = GetConfig();
            ReturnUrl = schemeHost + config["ReturnUrl"];
            CancelUrl = schemeHost + config["CancelUrl"];
            LogoUrl = schemeHost + config["LogoUrl"];
            SellerEmail = config["SellerEmail"];
            RedirectUrl = config["RedirectUrl"];
        }
      
        // Logs output statements, errors, debug info to a text file
        private static ILog logger = LogManager.GetLogger(typeof(SetExpressCheckout));

        // Create the configuration map that contains mode and other optional configuration details.
        public static Dictionary<string, string> GetConfig()
        {
            return PayPal.Manager.ConfigManager.Instance.GetProperties();
        }

        protected void Page_Load(object sender, EventArgs e)
        {
            HttpContext CurrContext = HttpContext.Current;
            // Create the SetExpressCheckoutResponseType object
            SetExpressCheckoutResponseType responseSetExpressCheckoutResponseType = new SetExpressCheckoutResponseType();
            try
            {
                //Get the query string parameters
                string item_name = Request.QueryString["item_name"];
                string item_id = Request.QueryString["item_id"];
                string item_desc = Request.QueryString["item_desc"];
                string item_quantity = Request.QueryString["item_quantity"];
                string item_amount = Request.QueryString["item_amount"];
                string tax_amount = Request.QueryString["tax_amount"];
                string shipping_amount = Request.QueryString["shipping_amount"];
                string handling_amount = Request.QueryString["handling_amount"];
                string shipping_discount_amount = Request.QueryString["shipping_discount_amount"];
                string insurance_amount = Request.QueryString["insurance_amount"];
                string total_amount = Request.QueryString["total_amount"];
                string currency_code = Request.QueryString["currency_code"];
                string payment_type = Request.QueryString["payment_type"];

                Session["SellerEmail"] = SellerEmail; 
                CurrencyCodeType currencyCode_Type = (CurrencyCodeType)Enum.Parse(typeof(CurrencyCodeType), currency_code, true);
                Session["currency_code_type"] = currencyCode_Type;
                PaymentActionCodeType payment_ActionCode_Type = (PaymentActionCodeType)Enum.Parse(typeof(PaymentActionCodeType), payment_type, true);
                Session["payment_action_type"] = payment_ActionCode_Type;
                // SetExpressCheckoutRequestDetailsType object
                SetExpressCheckoutRequestDetailsType setExpressCheckoutRequestDetails = new SetExpressCheckoutRequestDetailsType();
                // (Required) URL to which the buyer's browser is returned after choosing to pay with PayPal.
                setExpressCheckoutRequestDetails.ReturnURL = ReturnUrl;
                //(Required) URL to which the buyer is returned if the buyer does not approve the use of PayPal to pay you
                setExpressCheckoutRequestDetails.CancelURL = CancelUrl;
                // A URL to your logo image. Use a valid graphics format, such as .gif, .jpg, or .png
                setExpressCheckoutRequestDetails.cppLogoImage = LogoUrl;
                // To display the border in your principal identifying color, set the "cppCartBorderColor" parameter to the 6-digit hexadecimal value of that color
                setExpressCheckoutRequestDetails.cppCartBorderColor = "0000CD";

                // Set Landing Page to "Billing" for "PayPal Credit" 
                setExpressCheckoutRequestDetails.LandingPage = (LandingPageType)Enum.Parse(typeof(LandingPageType), "Billing", true);

                // Set Funding Source Type to "BML" for "PayPal Credit"
                FundingSourceDetailsType fundingSourceDetails = new FundingSourceDetailsType();
                fundingSourceDetails.UserSelectedFundingSource = (UserSelectedFundingSourceType)Enum.Parse(typeof(UserSelectedFundingSourceType), "BML", true);
                setExpressCheckoutRequestDetails.FundingSourceDetails = fundingSourceDetails;
               
                //Item details
                PaymentDetailsItemType itemDetails = new PaymentDetailsItemType();
                itemDetails.Name = item_name;
                itemDetails.Amount = new BasicAmountType(currencyCode_Type, item_amount);
                itemDetails.Quantity = Convert.ToInt32(item_quantity);
                itemDetails.Description = item_desc;
                itemDetails.Number = item_id;

                //Add more items if necessary by using the class 'PaymentDetailsItemType'

                // Payment Information
                List<PaymentDetailsType> paymentDetailsList = new List<PaymentDetailsType>();

                PaymentDetailsType paymentDetails = new PaymentDetailsType();
                paymentDetails.PaymentAction = payment_ActionCode_Type;
                paymentDetails.ItemTotal = new BasicAmountType(currencyCode_Type, item_amount);//item amount
                paymentDetails.TaxTotal = new BasicAmountType(currencyCode_Type, tax_amount); //tax amount;
                paymentDetails.ShippingTotal = new BasicAmountType(currencyCode_Type, shipping_amount); //shipping amount
                paymentDetails.HandlingTotal = new BasicAmountType(currencyCode_Type, handling_amount); //handling amount
                paymentDetails.ShippingDiscount = new BasicAmountType(currencyCode_Type, shipping_discount_amount); //shipping discount
                paymentDetails.InsuranceTotal = new BasicAmountType(currencyCode_Type, insurance_amount); //insurance amount
                paymentDetails.OrderTotal = new BasicAmountType(currencyCode_Type, total_amount); // order total amount
                
              
                paymentDetails.PaymentDetailsItem.Add(itemDetails);

                // Unique identifier for the merchant. 
                SellerDetailsType sellerDetails = new SellerDetailsType();
                sellerDetails.PayPalAccountID = SellerEmail;
                paymentDetails.SellerDetails = sellerDetails;

                paymentDetailsList.Add(paymentDetails);
                setExpressCheckoutRequestDetails.PaymentDetails = paymentDetailsList;

                SetExpressCheckoutReq setExpressCheckout = new SetExpressCheckoutReq();
                SetExpressCheckoutRequestType setExpressCheckoutRequest = new SetExpressCheckoutRequestType(setExpressCheckoutRequestDetails);
                setExpressCheckout.SetExpressCheckoutRequest = setExpressCheckoutRequest;

                // Create the service wrapper object to make the API call
                PayPalAPIInterfaceServiceService service = new PayPalAPIInterfaceServiceService();


                // API call
                // Invoke the SetExpressCheckout method in service wrapper object
                responseSetExpressCheckoutResponseType = service.SetExpressCheckout(setExpressCheckout);

                if (responseSetExpressCheckoutResponseType != null)
                {
                    // Response envelope acknowledgement
                    string acknowledgement = "SetExpressCheckout API Operation - ";
                    acknowledgement += responseSetExpressCheckoutResponseType.Ack.ToString();
                    logger.Debug(acknowledgement + "\n");
                    System.Diagnostics.Debug.WriteLine(acknowledgement + "\n");
                    // # Success values
                    if (responseSetExpressCheckoutResponseType.Ack.ToString().Trim().ToUpper().Equals("SUCCESS"))
                    {
                        // # Redirecting to PayPal for authorization
                        // Once you get the "Success" response, needs to authorise the
                        // transaction by making buyer to login into PayPal. For that,
                        // need to construct redirect url using EC token from response.
                        // For example,
                        // `redirectURL="https://www.sandbox.paypal.com/cgi-bin/webscr?cmd=_express-checkout&token="+setExpressCheckoutResponse.Token;`
                        // Express Checkout Token
                        string EcToken = responseSetExpressCheckoutResponseType.Token;
                        logger.Info("Express Checkout Token : " + EcToken + "\n");
                        System.Diagnostics.Debug.WriteLine("Express Checkout Token : " + EcToken + "\n");
                        // Store the express checkout token in session to be used in GetExpressCheckoutDetails & DoExpressCheckout API operations
                        Session["EcToken"] = EcToken;
                        Response.Redirect(RedirectUrl + "&token=" + EcToken, false);
                        Context.ApplicationInstance.CompleteRequest();
                    }
                    // # Error Values
                    else
                    {
                        List<ErrorType> errorMessages = responseSetExpressCheckoutResponseType.Errors;
                        string errorMessage = "";
                        foreach (ErrorType error in errorMessages)
                        {
                            logger.Debug("API Error Message : " + error.LongMessage);
                            System.Diagnostics.Debug.WriteLine("API Error Message : " + error.LongMessage + "\n");
                            errorMessage = errorMessage + error.LongMessage;                           
                        }
                        //Redirect to error page in case of any API errors
                        CurrContext.Items.Add("APIErrorMessage", errorMessage);
                        Server.Transfer("~/Response.aspx");                        
                    }
                }
            }
            catch (System.Exception ex)
            {
                // Log the exception message
                logger.Debug("Error Message : " + ex.Message);
                System.Diagnostics.Debug.WriteLine("Error Message : " + ex.Message);
            }
         }
    }
}